#!/usr/local/bin/python
'''
Created on Jun 6, 2011
@author: rchen
Create a bunch of extrinsicObjects that the registry uses. No objects.
'''

import sys, getopt, re
import time
from pds4regURL import pds4regURL

lidBase = "stressTesting"
urlBase = "http://localhost:8080/registry/extrinsics"
numProducts = 3
doDelete = True
doRegister = True
doView = True
viewLite = False
# keep the following 3 vars synched
templateLID  = "__lid__.T__counter__"	# needed to view
templateGUID = "__lid__.T__counter__.v1.0"	# needed to delete
templateRegister = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<extrinsicObject xmlns="http://registry.pds.nasa.gov"
    lid="__lid__.T__counter__"
    guid="__lid__.T__counter__.v1.0"
    versionId="1.0"
    name="Stress The Registry "
    objectType="Product"
    description="Stolen from http://pdscm/2010/registry/registry-service/operate/index.html with attribute lid above changed for uniqueness" >
    <slot name="first-name">
        <value>John</value>
    </slot>
    <slot name="last-name">
        <value>Doe</value>
    </slot>
    <slot name="phone">
        <value>(818)777-7777</value>
        <value>(818)888-8888</value>
    </slot>
    <slot name="cannotPossibleBeAnExistingSlot">
        <value>cannot possibly be an existing value</value>
    </slot>
</extrinsicObject>"""

def getStats(list):
	"given a list of numbers, return average, median, standard dev as a tuple"
	total = sum(list)	# length might reach 1000000, so do this once
	length = len(list)
	avg = total * 1.0 / length
	sList = sorted(list)
	med = sList[(length+1)/2 -1]	# works for list with odd length
	if length % 2 == 0:
		med = (sList[length/2-1] + sList[length/2]) / 2.0
	sumDiffSquared = 0
	for n in range(length):
		sumDiffSquared += (list[n]-avg)**2
	sd = (sumDiffSquared / length)**0.5
	return length, avg, med, sd, total

def usage():
	print "usage: ", sys.argv[0], "[-hDRVvn:l:]"
	print "  -h help"
	print "  -D do NOT delete from registry (happens before registering)"
	print "  -R do NOT register"
	print "  -V do NOT view (after registering)"
	print "  -v view but return shortened output"
	print "  -n<number> number of products to register. Default: %d" % numProducts
	print "  -l<lid> base of logical ID. Default: " + lidBase
	print "  -u<url> base of registry. Default: " + urlBase
	sys.exit()

try:
	opts, args = getopt.getopt(sys.argv[1:], "hDRVvn:l:u:",
		["help", "noDelete", "noRegister", "noView", "viewLite", "numProducts", "lid", "url"])
except getopt.GetoptError, err:
	print str(err)	  #will print something like "option -a not recognized"
	usage()
for o, a in opts:
	if o in ("-h", "--help"):
		usage()
	elif o in ("-D", "--noDelete"):
		doDelete = False
	elif o in ("-R", "--noRegister"):
		doRegister = False
	elif o in ("-V", "--noView"):
		doView = False
	elif o in ("-v", "--viewLite"):
		viewLite = True
	elif o in ("-n", "--numProducts"):
		numProducts = int(a)
	elif o in ("-l", "--lid"):
		lidBase = a
	elif o in ("-u", "--url"):
		urlBase = a
	else:
		usage()
if len(args) != 0: usage()
myX = pds4regURL(urlBase)

#if __name__ == "__main__":
if doDelete:
	timeDgood = []
	timeDbad = []
	t2 = templateGUID.replace("__lid__", lidBase)
	for i in range(numProducts):
		t3 = t2.replace("__counter__", "%06d" % i)
		time0 = time.time()
		try: ret = myX.lidDelete(t3)
		except: timeDbad.append(time.time() - time0)
		else:
			timeDgood.append(time.time() - time0)
			print ret
	if len(timeDgood) > 0:
		print "Delete %6d done. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeDgood)
	if len(timeDbad) > 0:
		print "Delete %6d FAIL. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeDbad)
if doRegister:
	timeRgood = []
	timeRbad = []
	t2 = templateRegister.replace("__lid__", lidBase)
	for i in range(numProducts):
		t3 = t2.replace("__counter__", "%06d" % i)
		time0 = time.time()
		try: ret = myX.lidRegister(t3)
		except: timeRbad.append(time.time() - time0)
		else:
			timeRgood.append(time.time() - time0)
			print ret
	if len(timeRgood) > 0:
		print "Regstr %6d done. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeRgood)
	if len(timeRbad) > 0:
		print "Regstr %6d FAIL. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeRbad)
if doView:
	timeVgood = []
	timeVbad = []
	t2 = templateLID.replace("__lid__", lidBase)
	for i in range(numProducts):
		t3 = t2.replace("__counter__", "%06d" % i)
		time0 = time.time()
		try: ret = myX.lidView(t3)
		except: timeVbad.append(time.time() - time0)
		else:
			timeVgood.append(time.time() - time0)
			if viewLite:
				m = re.search("lid\s*=\s*\"(.*?)\"", ret)
				if m: ret = "..." + m.group(1) + "..."
				else: ret = ret[0:50]
			print ret
	if len(timeVgood) > 0:
		print "View   %6d done. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeVgood)
	if len(timeVbad) > 0:
		print "View   %6d FAIL. Time(sec): avg=%.3f median=%.3f stdDev=%.5f sum=%.1f" % getStats(timeVbad)

